/* ---------------------------------------------------------------- */
/* LICENSED MATERIALS PROPERTY OF IBM                               */
/*                                                                  */
/* (C)Copyright IBM Corp.  2003                                     */
/*                                                                  */
/* All rights reserved                                              */
/*                                                                  */
/* US Government Users Restricted Rights -                          */
/* Use, duplication or disclosure restricted by                     */
/* GSA ADP Schedule Contract with IBM Corp.                         */
/*  --------------------------------------------------------------- */
/*                                                                  */
//////////////////////////////////////////////////////////////////////
//
//                          DISCLAIMER
//                          ----------
//
//This material contains IBM copyrighted sample programming source code.
//IBM grants you a nonexclusive copyright license to use the Sample
//Code as an example from which you can generate similar function
//tailored to your own specific needs.  Your license to this Sample
//Code provides you no right or licenses to any IBM patents.
//
//The Sample Code is provided by IBM for illustrative purposes only.
//The Sample Code has not been thoroughly tested under all conditions.
//IBM, therefore, cannot guarantee or imply its reliability, serviceability,
//performance or function.  All Sample Code contained herein
//is provided to you "AS IS".  THE IMPLIED WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
//AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
//SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION 
//OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSIONS 
//MAY NOT APPLY TO YOU. IN NO EVENT WILL IBM BE LIABLE 
//TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR 
//OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF 
//THE SAMPLE CODE INCLUDING, WITHOUT LIMITATION, 
//ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS OF 
//PROGRAMS OR OTHER DATA ON YOUR INFORMATION 
//HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE 
//EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH 
//DAMAGES. 
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
//
// DirCleanerAgent
//
// This is a finished example agent that is used in the ABLE
// lab AbleEServerAgentsLab.doc.
//
// This agent searches a directory and its subdirectories for
// large files and deletes them.
//

package agent;

import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.HashMap;

import com.ibm.able.Able;
import com.ibm.able.AbleException;
import com.ibm.able.AbleMessageContainer;
import com.ibm.able.platform.AbleAgentSituationReport;
import com.ibm.able.platform.AbleEServerCapability;
import com.ibm.able.platform.AbleEServerDefaultAgent;
import com.ibm.able.platform.AbleEServerSituation;
import com.ibm.able.platform.AblePlatform;
import com.ibm.able.platform.AbleSecureKey;

/**
 * Directory cleaner agent.
 */
public class DirCleanerAgent extends AbleEServerDefaultAgent
{

    //------------------------------------------------------------
    // Capabilities
    //------------------------------------------------------------


    static public final AbleMessageContainer CAP_CLEANUP_NAME = new AbleMessageContainer("Cleanup files");
    static public final Integer CAP_CLEANUP_ID = new Integer(1);
    
    //------------------------------------------------------------
    // Situations
    //------------------------------------------------------------

    static public final AbleMessageContainer SIT_FOUND_LARGE_FILE_NAME = 
    							new AbleMessageContainer("File {0} is {1} bytes.");
	static public final Integer SIT_FOUND_LARGE_FILE_ID = new Integer(2);
	
    //------------------------------------------------------------
    // Responses
    //------------------------------------------------------------

    static public final AbleMessageContainer RESP_DELETE_FILE = new AbleMessageContainer("Delete the file.");
    static public final AbleMessageContainer RESP_SKIP_FILE = new AbleMessageContainer("Skip this one file.");

	
	//----------------------------------------------------------------
	// Other fields
 	//----------------------------------------------------------------
    
    private DirScanner m_scanner = null;
    private int m_numChecked = 0;
    private int m_numDeleted = 0;
    private String m_dirName = "C:\\temp";
    
	//----------------------------------------------------------------
	// Constructors
	//----------------------------------------------------------------

	/**
	 * Default no-arg constructor
	 */
	public DirCleanerAgent() throws RemoteException, AbleException
	{
    	this("MyDirCleanerAgent");
	}
	
	/**
	 * Takes the agent display name.  Creates a new DirCleanerAgent.
	 */
	public DirCleanerAgent(String displayName) throws RemoteException, AbleException
	{
		// Call super constructor and give it agent's display name
		super(displayName);
		
		// Set agent type that shows up in the Type column of the Console
		setAgentType("Directory Cleaner");
		
        // Create the DirScannerBean and add it to this agent.
        m_scanner = new DirScanner(displayName + "-scannerbean");
        m_scanner.setRootName(m_dirName);
        addBean(m_scanner);
        
        setupCapabilities();
		
	}
	
	/**
	 * Overrides lclInit(Object) in AbleEServerDefaultAgent.  Gets called
	 * when agent is initialized (when user presses Start button).
	 */
	public void lclInit(Object capabilities) throws AbleException
    {

        // Enable timer processing, but not asynch events or data flow
        setTimerEventProcessingEnabled(true);
        setSleepTime(1000);   // call processTimerEvent() every second
        setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);
        setDataFlowEnabled(false);
        
        // Make the DirScanner start over with root dir.
        m_scanner.init();
        startEnabledEventProcessing();
        super.lclInit(capabilities);
    }

	/**
	 * Called at user-set time intervals.
	 */
	public void processTimerEvent()
	{
	    // call process().  Each time process called, one more file name ran
        // through the ruleset in this agent.
	    process();	
	}


	/**
	 * Does agent's actual work.  Called at certain time interval 
	 * from processTimerEvent().
	 */
	public void process()
	{
	    try
	    {

            // Calls process() on beans inside of agent. (scanner)
            super.process();
            
            m_numChecked++;
            
            // get output vars from scanner
            String [] fileName = (String[]) m_scanner.getOutputBuffer();
            
			// If a pathname was returned, check file size.
			if (! fileName[0].equals(""))
			{

				long fileSize = Long.parseLong(fileName[1]);
				if (fileSize > 500)
				{
					if (trace.isLogging()) trace.text(Able.TRC_LOW, "File: " + fileName[0] + "  size: " + fileName[1]);
					largeFileFound(fileName);
				}
			}


	    } catch (RemoteException re)
	    {
	        if (trace.isLogging()) trace.entry(Able.TRC_LOW, this, "Error in DirCleanerAgent.process()");
	    }
	}

    private void largeFileFound(String [] text)
    {
        String lclM="foundLargeFile()"; 
        
        try
        {
        
            //Begin to create the AbleAgentSituation report for this capabilitie's situation
            AbleEServerCapability capability = (AbleEServerCapability)capabilities.get(CAP_CLEANUP_ID);
            AbleEServerSituation situation = (AbleEServerSituation)capability.getSituations().get(SIT_FOUND_LARGE_FILE_ID);

            // Going to replace the situation string with one that has the name of the file in it.
            AbleMessageContainer newSitName = new AbleMessageContainer(MessageFormat.format(situation.getName().getMessage(),text));
        
            // Make a new AbleEServerSituation with the formatted text.
            AbleEServerSituation newSit = new AbleEServerSituation(situation.getSituationIdentifier(),
                                                                    newSitName,
                                                                    situation.getSeverityLevel(),
                                                                    situation.getSituationType(),
                                                                    situation.getResponses());
        
            // Create the ASR that is sent to the logging service.
            AbleAgentSituationReport asr = new AbleAgentSituationReport(text[0],
                                                                        capability,
                                                                        newSit,
                                                                        situation.getYesResponse(),
                                                                        myJasAgentName,  
                                                                        getName(),
                                                                        System.currentTimeMillis(),
                                                                        getAgentHost(),
                                                                        new Integer(getAgentAutonomyLevel())); 

            switch (capability.getAutonomyLevel())
            {       
                case AbleEServerCapability.YES_CAPABILITY:
                {
                    //Log a history log telling that the agent is acting on it's own
                    logHistory(new AbleMessageContainer("Agent is taking own action"),
                               new AbleMessageContainer("The "+capability.getName()+" capability is set to YES."),
                               new AbleMessageContainer("Will be attempting to: "+situation.getYesResponse()));

                    asr.setProblemState(false);
                    asr.setResponseTaken(situation.getYesResponse());

                    actOnLargeFile(asr);

                    logAbleAgentSituationReport(asr);
                    return;
                
                }

                case AbleEServerCapability.ASK_CAPABILITY:
                {
                    //Log the actual request, and then another history logs saying that a problem had occured
                    asr.setProblemState(true);
                    logAbleAgentSituationReport(asr);

                    logHistory(new AbleMessageContainer("A request has been logged"),
                           new AbleMessageContainer("Capability: "+asr.getCapability().getName()+
                                                    ", Situation: "+asr.getSituation().getName()),
                           new AbleMessageContainer("Will be waiting for an answer"));
                    return;
                }

                case AbleEServerCapability.NO_CAPABILITY:
                {
            
                    asr.setProblemState(false);
                    asr.setResponseTaken(situation.getNoResponse());
                    asr.setResultsSummary(new AbleMessageContainer("Agent is not taking any action."));
                    logAbleAgentSituationReport(asr);
                    return;
                }               

            }
        } catch (AbleException ae)
        {
            if (trace.isLogging()) trace.entry(Able.TRC_LOW, this, "AbleException in foundLargeFile(): " + ae.getMessage());
        }
    }
    
    /**
	 * Sets the capabilities of this agent.
	 * 
	 * We have one capability, to cleanup large files
	 */
	public void setupCapabilities() throws AbleException
	{
        //------------------------------------------------------------
        // 1:  Set up capabilities/situations
        //------------------------------------------------------------

   		//Define the first situation for this "Cleanup" capability: SIT_FOUND_LARGE_FILE
   		AbleMessageContainer[] situation1Responses 
   					= new AbleMessageContainer[]{RESP_DELETE_FILE, RESP_SKIP_FILE};  
		AbleEServerSituation situation1 = new AbleEServerSituation(SIT_FOUND_LARGE_FILE_ID,
												SIT_FOUND_LARGE_FILE_NAME, AbleEServerSituation.LOW_SEVERITY,
												AbleEServerSituation.PERMISSION_REQUEST, situation1Responses);
		
		//Create a Map to hold the capability of this agent
		HashMap situations = new HashMap();
		
		situations.put(SIT_FOUND_LARGE_FILE_ID, situation1);

		AbleEServerCapability cleanupCap = new AbleEServerCapability(CAP_CLEANUP_ID, CAP_CLEANUP_NAME,
														AbleEServerCapability.ASK_CAPABILITY, situations);
        //------------------------------------------------------------
        // 2: Put capabilities into Map
        //------------------------------------------------------------

   		capabilities.clear();
   	 	capabilities.put(CAP_CLEANUP_ID, cleanupCap);

        //------------------------------------------------------------
        // 3: Define the presets
        //------------------------------------------------------------

   		highCapabilitySettings = new HashMap();
   		highCapabilitySettings.put(CAP_CLEANUP_ID, new Integer(AbleEServerCapability.YES_CAPABILITY));
   		mediumCapabilitySettings = new HashMap();
   		mediumCapabilitySettings.put(CAP_CLEANUP_ID, new Integer(AbleEServerCapability.ASK_CAPABILITY));
   		lowCapabilitySettings = new HashMap();
   		lowCapabilitySettings.put(CAP_CLEANUP_ID, new Integer(AbleEServerCapability.NO_CAPABILITY));

        //------------------------------------------------------------
		// 4: For checking purposes, and to init the custom capability settings, verify the capabilities
		//------------------------------------------------------------
		verifyCapabilities();
	}

    /**
     * Takes action after the user responds to the request.
     * 
     * @param asr
     * @throws AbleException
     */
    private void actOnLargeFile(AbleAgentSituationReport asr) throws AbleException
    {

        // Get file name (saved in Identifier in ASR)
        String fileName = (String) asr.getIdentifier();
        
        // Delete the file
        if(asr.getResponseTaken().equals(RESP_DELETE_FILE))
        {
            m_numDeleted++;

            if (trace.isLogging()) trace.text(Able.TRC_LOW, "*** deleting file " + fileName);

            // Add code here to actually delete the file
            
            asr.setResultsSummary(new AbleMessageContainer("File deleted."));
        
        }

        // Skipping this file.
        else if(asr.getResponseTaken().equals(RESP_SKIP_FILE))
        {
            asr.setResultsSummary(new AbleMessageContainer("Skipping this file."));
        
            // Do nothing.
        }

        else  //invalid response taken in the Agent Situation Report
            asr.setResultsSummary(new AbleMessageContainer("The response give: "+asr.getResponseTaken()+
                                                            " is not a valid response"));
            
    }    
    


    /**
     * @see com.ibm.able.platform.AbleEServerDefaultAgent#_answerRequest(AbleAgentSituationReport, AbleSecureKey)
     */
    protected void _answerRequest(AbleAgentSituationReport agentReport, AbleSecureKey key)
    {

        try
        {
            actOnLargeFile(agentReport);    
            
            //update the asr with the logging service
            updateAsrAfterAction(agentReport);
        
            //update the summary string of this agent to reflect what it has done
            updateAgentSummaryString();


        }
        
        //An error occured answering this situation... log a history log describing this error
        catch(AbleException ae)
        {
            logException("_answerRequest()", ae);                 
        }           
    }	

    /**
     * After actions have been taken, this method has the Logging Service update this ASR. 
     * At this point, the only thing that should be different in the ASR from the logging 
     * service is the result summary field, and responseTaken.
     * 
     * @param asr The AbleAgentSituationReport that we will have the Logging Service update
     * @throws AbleException    
     */
    private void updateAsrAfterAction(AbleAgentSituationReport asr) throws AbleException
    {
        String lclM = "updateAsrAfterAction";
        //Create a new asr that will represent the old ASR that should still exist in the logging service
        AbleAgentSituationReport oldAsr = new AbleAgentSituationReport(asr);
        oldAsr.setResultsSummary(null);
        oldAsr.setResponseTaken(null);

        //Mark the new ASR as being a history log now that we've had it answered
        asr.setProblemState(false);
        
        updateAbleAgentSituationReport(oldAsr,asr);
    }
    
    /**
     * Part of the AblePlatformDefaultAgent is the summary of the agent.  This is just an quick overview
     * of what the agent has done.  Whenever we do something, this should be updated, so when someone
     * at the console is viewing the agent, they can look at the summary and get an idea of what this
     * agent has done
     */
    private void updateAgentSummaryString() throws AbleException
    {
        setAgentSummary(new AbleMessageContainer("Files checked: "+ m_numChecked +", files deleted: "+ m_numDeleted));
        
        //Have the agent update its description with this new information.
        myJasAgentDescription.set(AblePlatform.AbleAgentSummary, getAgentSummary());
        updateDescriptionWithAgentDirectoryService();       
    }



}
