/* ---------------------------------------------------------------- */
/* LICENSED MATERIALS PROPERTY OF IBM                               */
/*                                                                  */
/* (C)Copyright IBM Corp.  2003                                     */
/*                                                                  */
/* All rights reserved                                              */
/*                                                                  */
/* US Government Users Restricted Rights -                          */
/* Use, duplication or disclosure restricted by                     */
/* GSA ADP Schedule Contract with IBM Corp.                         */
/*  --------------------------------------------------------------- */
/*                                                                  */
//////////////////////////////////////////////////////////////////////
//
//                          DISCLAIMER
//                          ----------
//
//This material contains IBM copyrighted sample programming source code.
//IBM grants you a nonexclusive copyright license to use the Sample
//Code as an example from which you can generate similar function
//tailored to your own specific needs.  Your license to this Sample
//Code provides you no right or licenses to any IBM patents.
//
//The Sample Code is provided by IBM for illustrative purposes only.
//The Sample Code has not been thoroughly tested under all conditions.
//IBM, therefore, cannot guarantee or imply its reliability, serviceability,
//performance or function.  All Sample Code contained herein
//is provided to you "AS IS".  THE IMPLIED WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
//AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
//SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION 
//OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSIONS 
//MAY NOT APPLY TO YOU. IN NO EVENT WILL IBM BE LIABLE 
//TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR 
//OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF 
//THE SAMPLE CODE INCLUDING, WITHOUT LIMITATION, 
//ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS OF 
//PROGRAMS OR OTHER DATA ON YOUR INFORMATION 
//HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE 
//EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH 
//DAMAGES. 
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
//
//
//====================================================================

package agent;

//====================================================================
// Imports
//====================================================================
import java.io.File;
import java.util.Vector;

import com.ibm.able.Able;
import com.ibm.able.AbleException;
import com.ibm.able.AbleObject;

/**
 *  This class is an example of a simple AbleBean created by extending
 *  AbleObject.  It scans a directory tree recursively, returning
 *  file names and sizes.
 *
 */
 public class DirScanner extends AbleObject
 {

    //==================================================================
    //
    // Class Variables
    //
    //==================================================================

    //==================================================================
    //
    // Instance Variables
    //
    //==================================================================

    // Directory that is being scanned.
    private String m_rootName = null;  
    
    // Keep track of subdirs to search.  When finish a directory,
    // Take a subdir from this list.
    private Vector m_subDirList = null; 

    // Read list of files from current directory, save here.    
    private String [] m_currentList = null;
    private String m_currentDir = null;
    private int m_index = 0;
                            
    private String [] outBufRef;
    
   
    //==================================================================
    //
    // Constructors
    //
    //==================================================================

    /**
     * Construct an DirScanner bean with a name of DirScanner.
     *
     * @return    DirScanner
     */
    public DirScanner() 
    {
        this("DirScanner");
    }

    /**
     * Construct an DirScanner with the provided bean name.
     * Bean will be in uninitiated state.
     *
     * @param     theBeanName A String containing the name of the object.
     * @return    IFSScanner.
     */
    public DirScanner(String theBeanName) 
    {
        super(theBeanName);

        // Create outBuffer
        outputBuffer = new String[4];
       
        outBufRef = (String []) outputBuffer;

        // Init/clear out m_openDirList vector
        m_subDirList = new Vector();

        // No Timer processing, and no asynch events
        setSleepTime(0);
        setTimerEventProcessingEnabled(false);
        setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);
        setDataFlowEnabled(true) ;
        
        // Start over with root path name.
        m_subDirList = new Vector();
        if ((m_rootName != null) && (! m_rootName.equals("")))
        {
        	m_subDirList.add(m_rootName);
        }
        
        // Clear out anything that might have been in currentList
        m_currentList = null;
        
    }
   
    //==================================================================
    //
    // Accessor methods for instance variables
    //
    //==================================================================

    /**
     * Set the root dir name to scan.
     *
     * @param theName 
     */
  
    public void setRootName(String theName)
    {
        setRootName(theName, false);
    }
  
  
    /**
     * Set the directory name.  The bean has to be in initialized state
     * when calling this--if you've already used the bean, you have to
     * call reset() first before calling this method.
     *
     * @param theName A String containing the path and name of the file to watch.
     * @param sendChange A boolean indicating whether to fire a property change.
     */
    private void setRootName(String theName, boolean sendChange)
    {
        String theOldName = m_rootName;
        m_rootName = theName;

        if (sendChange) firePropertyChange("dirScannerRootName", theName, theOldName);

        // Start over with root path name.
        m_subDirList = new Vector();
        if ((m_rootName != null) && (! m_rootName.equals("")))
        {
        	m_subDirList.add(m_rootName);
        }
        
        // Clear out anything that might have been in currentList
        m_currentList = null;

    }
  
    /**
     *
     */
    public String getRootName() { return m_rootName; }
  
    //==================================================================
    //
    // Methods to override AbleObject
    //
    //==================================================================


    /**
     * Reset the bean to a known initialized state.
     *
     */
    public void reset()
    {

        // No Timer processing, and no asynch events
        setSleepTime(0);
        setTimerEventProcessingEnabled(false);
        setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);
        setDataFlowEnabled(true) ;
        
        // Start over with root path name.
        m_subDirList = new Vector();
        if ((m_rootName != null) && (! m_rootName.equals("")))
        {
        	m_subDirList.add(m_rootName);
        }
        
        // Clear out anything that might have been in currentList
        m_currentList = null;
        
    }

    /**
     * Initialize bean. Should only be called once.  Doesn't set root
     * name, have to call setRootName first.  Opens rootName, clears
     * out any other open directories.
     */
    public void init() throws AbleException
    {
      
        // locals
        // Make sure rootName already set.
        if(m_rootName == null)
        {
            throw new AbleException("Have to set rootName property before initializing DirScanner.");
        }
      
        // Check to make sure root directory is actually a directory.
        File rootDir = new File(m_rootName);
        
        // Make sure this is a directory.
        if (! rootDir.isDirectory())
        {
            throw new AbleException("Root dir has to be a directory.");
        }
        
        // Add root dir to list of subdirs to search.
        m_subDirList.add(m_rootName);
        
        // Clear out anything that might have been in currentList
        m_currentList = null;

        // Everything worked, so call superclass init.  Sets state.
        super.init();
    }

    /**
     * Return the next file in m_currentList.  If this hits any directories,
     * it just adds them to m_subDirList and keeps going until it finds
     * a normal file.  If runs out of entries in m_currentList, returns null.
     */
    private File nextFileFromCurrentList()
    {

        while (m_index < m_currentList.length)
        {
            // Grab the next file from list
            File nextFile = new File(m_currentDir + File.separator + m_currentList[m_index]);
        
            // If it is a directory, add it to the list of subdirs
            // to search.
            if (nextFile.isDirectory())
            {
                m_subDirList.add(m_currentDir + File.separator + m_currentList[m_index]);
            
                // Grab the next file from list.
                m_index++;
                continue;
            }
            
            if (nextFile.isFile())
            {
            	m_index++;
            	return nextFile;
            }
            
            m_index++;
        }
        
        return null;
    }
    
    /**
     * readNextSubDir().  called to grab another subDir from m_subDirList,
     * open it and read it.  Returns array of string path names.
     */
    private String [] readNextSubDir()
    {
    	m_index = 0;
    	
        // Get next subdir from list.
        while (m_subDirList.size() > 0)
        {
            // Take most recently added subdirs.
            m_currentDir = (String) m_subDirList.remove(m_subDirList.size() - 1);
            File nextSubdir = new File(m_currentDir);
            String [] nextCurList = nextSubdir.list();
            
            // If can't get anything from this directory, just skip
            // to next subdir.
            if ((nextCurList == null) || (nextCurList.length == 0))
            {
            	continue;
            }
            
            return nextCurList;
        }

        // If there aren't anymore subdirs, return null.        
        m_currentDir = null;
        return null;
    }

    /**
     *  Each time nextFile() called, return next file in dir tree.
     *  If no more files, returns null.
     *
     * @exception An exception is thrown if an action fails.
     */
    private File nextFile() 
    {
  	
        // Set up m_currentList, m_index if they aren't set up yet.
        if(m_currentList == null)
        {
        	m_currentList = readNextSubDir();

        	if (m_currentList == null)
        	{
        		return null;
        	}
        }
        
        // Get next file.
        File nextFile = nextFileFromCurrentList();
        
        while (nextFile == null)
        {
        
        	m_currentList = readNextSubDir();
        	if (m_currentList == null)
        	{
        		return null;
        	}

            nextFile = nextFileFromCurrentList();
        }
        
        return nextFile;

    }
    
    /**
     * process()
     */
    public void process()
    {    
        
        File nextFile = null;

        try
        {       
            nextFile = nextFile();
        } catch (Exception e)
        {
            if (Able.TraceLog.isLogging()) Able.TraceLog.text(Able.TRC_LOW, this, "Caught exception:", e.getMessage());
        	
        }
       
        if (nextFile != null)
        {
            String objName = nextFile.getAbsolutePath();
        
        
            outBufRef[0] = objName;
            outBufRef[1] = Long.toString(nextFile.length());
        }
        else
        {
            outBufRef[0] = "";
            outBufRef[1] = "";
        }
    }

    //==================================================================
    //
    // Miscellaneous methods
    //
    //==================================================================

    /**
     * Determine the copyright of this class.
     *
     * @return    String A String containing this class's copyright statement.
     *            <p>
     */
  
    public static String Copyright()
    {
        return Able.Copyright;
    }
    
    public static void main(String [] args)
    {
    	
        String outFileName = null;

    	try
    	{
    		
            // Create a new DirScanner bean.
	    	DirScanner scanner = new DirScanner("Scanner bean 1");
    	
	    	scanner.setRootName("C:\\");
    	
	    	scanner.init();

    	
    		do
    		{
	    	    scanner.process();

	            // get output vars from sorter (ruleset).
    	        String []test = (String []) scanner.getOutputBuffer();
    
    		    outFileName = test[0];
    	    
    		    System.out.println(outFileName + " " + test[1]);
	    	} while (! outFileName.equals(""));
    	} catch (Exception e)
    	{
   		    System.out.println(e.toString());
    	}
    }
  
  
}

