/* ---------------------------------------------------------------- */
/* LICENSED MATERIALS PROPERTY OF IBM                               */
/*                                                                  */
/* (C)Copyright IBM Corp.  2003                                     */
/*                                                                  */
/* All rights reserved                                              */
/*                                                                  */
/* US Government Users Restricted Rights -                          */
/* Use, duplication or disclosure restricted by                     */
/* GSA ADP Schedule Contract with IBM Corp.                         */
/*  --------------------------------------------------------------- */
/*                                                                  */
//////////////////////////////////////////////////////////////////////
//
//                          DISCLAIMER
//                          ----------
//
//This material contains IBM copyrighted sample programming source code.
//IBM grants you a nonexclusive copyright license to use the Sample
//Code as an example from which you can generate similar function
//tailored to your own specific needs.  Your license to this Sample
//Code provides you no right or licenses to any IBM patents.
//
//The Sample Code is provided by IBM for illustrative purposes only.
//The Sample Code has not been thoroughly tested under all conditions.
//IBM, therefore, cannot guarantee or imply its reliability, serviceability,
//performance or function.  All Sample Code contained herein
//is provided to you "AS IS".  THE IMPLIED WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
//AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
//SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION 
//OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSIONS 
//MAY NOT APPLY TO YOU. IN NO EVENT WILL IBM BE LIABLE 
//TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR 
//OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF 
//THE SAMPLE CODE INCLUDING, WITHOUT LIMITATION, 
//ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS OF 
//PROGRAMS OR OTHER DATA ON YOUR INFORMATION 
//HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE 
//EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH 
//DAMAGES. 
//
//////////////////////////////////////////////////////////////////////

package com.ibm.able.examples.ableagent;

//====================================================================
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// Product: com.ibm.able.examples.ableagent
//
// (C) Copyright IBM Corp. 2000 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
//
//                             DISCLAIMER
//                             ----------
//
// This material contains programming source code for your consideration.
// These examples have not been thoroughly tested under all conditions.
// IBM, therefore, cannot guarantee or imply reliability, serviceability,
// performance or function of these programs.  All programs contained
// herein are provided to you "AS IS".  THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY
// DISCLAIMED.
//
//====================================================================

//====================================================================
// Imports
//====================================================================

import java.rmi.RemoteException;

import com.ibm.able.Able;
import com.ibm.able.AbleDefaultAgent;
import com.ibm.able.AbleException;

/**
 * This class is an example of an AbleAgent created by extending
 * AbleDefaultAgent.  It uses timer event processing to set a new value
 * in its outputBuffer, and sends a data event
 * indicating its outputBuffer contents have changed.
 *
 * @see com.ibm.able.agents.AbleDefaultAgent
 *
 * @author   Jeff Awe
 */
public class StoragePredictionAgent extends AbleDefaultAgent 
{
  //==================================================================
  //
  // Instance Variables
  //
  //==================================================================

  // define any bean-specific instance variables here
  private AbleDefaultAgent StoragePrediction;
  private String   Prediction;
  private double[] agentPrediction;        

  //==================================================================
  //
  // Constructors
  //
  //==================================================================

  /**
   * Construct a simple AbleAgent and initialize it.
   *
   */
  public StoragePredictionAgent() throws RemoteException 
  {
      this("SimpleAgent");
      
  }


  /**
   * Construct a simple AbleAgent with specified name.
   *
   * @param     aName
   *            A String object specifying the object name
   */
  public StoragePredictionAgent(String aName) throws RemoteException 
  {
       super(aName) ;
       
       // set agent defaults
       // need to allocate the input and output buffers here (if used)
       inputBuffer = new String[1] ;
       outputBuffer = new String[1] ;
       reset();
       // in the case of this example, start processing
       init(); 
  }


  //------------------------------------------------------------------
  // Processing methods
  //------------------------------------------------------------------
  /**
   * Initialize the agent to a known initialized state.
   * Start processing any events enabled.
   *
   */
  public void init() throws AbleException 
  {
       if (trace.isLogging()) trace.text(Able.TRC_LOW,this,"init","Prediction = " + this.Prediction);
       super.init();  // create/resume the  event queue thread
  }


  /**
   * Reset the agent to default processing settings.
   *
   */
  public void reset() throws AbleException 
  {
       setDataFlowEnabled(true) ;
       // Timer processing, but no asynch events
       setSleepTime(5000);   // call processTimerEvent() every 5 seconds
       setTimerEventProcessingEnabled(true);
       setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);
  }


  /**
   * Perform the standard processing function performed by this agent.
   * This typically takes data from the inputBuffer, processes it,
   * and places it in the output buffer.
   *
   * @exception AbleException
   *            either RemoteException or process error
   */
  public void process() throws AbleException 
  {
         processBufferConnections() ;   // move data into inputBuffer (if any)

         // perform the main processing here
         Object in = getInputBuffer(0) ;  // see if there is data in the input buffer
         if (in != null) 
         {
           setStoragePrediction((String)getInputBuffer(0)) ; // copy to the prediction parameter
         } 
         else 
         { 
           //If you really wanted to use the storage prediction agent, this is the place
           //where you would set the inputs with current information from the system.
           //In this case, everytime it runs it uses these inputs	
            double Lib1NumberOfFiles  	=152;
		double Lib1Size  			=3452;
		double Lib2NumberOfFiles  	=284;
		double Lib2Size  			=4444;
		double Lib3NumberOfFiles  	=226;
		double Lib3Size  			=3667;
		double Lib4NumberOfFiles  	=95;
		double Lib4Size  			=477;
		double Lib5NumberOfFiles  	=656;
		double Lib5Size  			=54978;
		double Lib6NumberOfFiles  	=45;
		double Lib6Size  			=5521;
		double Lib7NumberOfFiles  	=13;
		double Lib7Size  			=999;
		double StorageUtilization 	=0;

	
           
           	double inputdoubleBuf[] = { 	Lib1NumberOfFiles, Lib1Size, Lib2NumberOfFiles,  	
			 				Lib2Size, Lib3NumberOfFiles, Lib3Size, Lib4NumberOfFiles,  
			                     	Lib4Size, Lib5NumberOfFiles, Lib5Size, Lib6NumberOfFiles,  	
			                       	Lib6Size, Lib7NumberOfFiles, Lib7Size, StorageUtilization};	           
         }    
         // Note: we either process data in the input buffer
         //       or use other data member values as inputs

         // assign output data to the outputBuffer here
         setOutputBuffer(0, agentPrediction[0]+"") ;

         dataChanged(outputBuffer) ;  // tell any AbleEventListeners that we changed
  }


  /**
  *   This method gets called from the asych thread (if running)
  *   Default behavior is to just call the process() method
  *
  */
   public void processTimerEvent() throws AbleException
   {
        process() ;
   }


  /**
  *   Set the prediction attribute
  *   Also fire a propertyChange event and a dataChanged AbleEvent
  */
  public void setStoragePrediction(String newPrediction) {
       String oldPrediction = Prediction;
       Prediction = newPrediction ;
       chgSupport.firePropertyChange("Prediction", oldPrediction, newPrediction);
  }

  /**
  *   Get the prediction attribute
  */
  public String getPrediction() { return Prediction; }

    /**
     * Determine the copyright of this class.
     *
     * @return    A String containing this class's copyright statement.
     *            <p>
     */
    public static String Copyright() {
      return Able.Copyright;
    }
    
    public static void main(String args[])
    {
    	try
    	{
    	  StoragePredictionAgent jeff = new StoragePredictionAgent();
    	}
    	catch(Exception e){}
    }
}
