/* ---------------------------------------------------------------- */
/* LICENSED MATERIALS PROPERTY OF IBM                               */
/*                                                                  */
/* Product(s):                                                      */
/*     5722-SS1                                                     */
/*                                                                  */
/* (C)Copyright IBM Corp.  2003                                     */
/*                                                                  */
/* All rights reserved                                              */
/*                                                                  */
/* US Government Users Restricted Rights -                          */
/* Use, duplication or disclosure restricted by                     */
/* GSA ADP Schedule Contract with IBM Corp.                         */
/*  --------------------------------------------------------------- */
/*                                                                  */
//////////////////////////////////////////////////////////////////////
//
//                          DISCLAIMER
//                          ----------
//
//This material contains IBM copyrighted sample programming source code.
//IBM grants you a nonexclusive copyright license to use the Sample
//Code as an example from which you can generate similar function
//tailored to your own specific needs.  Your license to this Sample
//Code provides you no right or licenses to any IBM patents.
//
//The Sample Code is provided by IBM for illustrative purposes only.
//The Sample Code has not been thoroughly tested under all conditions.
//IBM, therefore, cannot guarantee or imply its reliability, serviceability,
//performance or function.  All Sample Code contained herein
//is provided to you "AS IS".  THE IMPLIED WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
//AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
//SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION 
//OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSIONS 
//MAY NOT APPLY TO YOU. IN NO EVENT WILL IBM BE LIABLE 
//TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR 
//OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF 
//THE SAMPLE CODE INCLUDING, WITHOUT LIMITATION, 
//ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS OF 
//PROGRAMS OR OTHER DATA ON YOUR INFORMATION 
//HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE 
//EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH 
//DAMAGES. 
//
//////////////////////////////////////////////////////////////////////
package com.ibm.able.platform.examples;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Vector;

import javax.agent.service.naming.NamingException;

import com.ibm.able.Able;
import com.ibm.able.AbleException;
import com.ibm.able.AbleMessageContainer;
import com.ibm.able.platform.AbleAgentSituationReport;
import com.ibm.able.platform.AbleEServerCapability;
import com.ibm.able.platform.AbleEServerDefaultAgent;
import com.ibm.able.platform.AbleEServerSituation;
import com.ibm.able.platform.AblePlatform;
import com.ibm.able.platform.AblePlatformPreferences;
import com.ibm.able.platform.AbleSecureKey;
import com.ibm.able.platform.AgentLoggingException;

/**
 * This class is a simple eServer agent example.
 * It is a time monitor agent.  It uses the basics of Able to continusouly monitor the time.
 * Everytime the minute or hour changes the agent will send a history log about the event.  
 * The logs could be in one of three formats, and the capability settings will decide how it will
 * act.  There will also be a second capability that will check for duplicate request logs.  
 * The interval that this agent will check the time at can be customized through a customization
 * panel.  
 * <p>
 * Depending on it's setting, everytime the hour or minute changes, the time capability will either
 * send a history log with no time info, do nothing at all, or ask what it should do.
 * When sending a request with what it should do, it will give the following options:  
 * <ul>
 *   <li>Send this as history to the log by not displaying the time at all.
 *   <li>Send this as history to the log by displaying the time as a long.
 *   <li>Send this as history to the log by displaying the time in the MMDDYY HHMMSS format.
 *   <li>Do nothing.
 * </ul>
 * Depending on it's setting, the duplicate request capability will never log duplicate requests,
 * always log duplicate requests, or ask if a duplicate request should be logged.
 * When the capability is set to ask, these will be the choices:
 * <ul>
 *   <li>Create a duplicate request.
 *   <li>Do not create a duplicate request.
 * </ul>
 * <p>
 * This class will extend AbleEServerDefaultAgent.  Extending this will give you 
 * what an agent needs to use the Able Admin Console.  It allows you to use the logging service
 * to send logs to, and to get these logs answered.  It gives you the capability functionality
 * so the agent can easily set this up and use it to determine how to handle situations.
 * Others can then make use of these too, to change them or just view them.  It also allows an
 * agent to have a customization panel that the admin console can use to customize the agent and
 * an about panel that will display information about the agent.
 * <p>
 * AbleEServerDefaultAgent contains the following variables that a subclass will use:
 *     AgentLoggingService  myEServerLoggingService
 *     HashMap              capabilities
 *     Vector               capabilityOrder
 *     HashMap              highCapabilitySettings
 *     HashMap              mediumCapabilitySettings
 *     HashMap              lowCapabilitySettings
 *     HashMap              customCapabilitySettings
 * <p>
 * The follwing methods will be used to work with capabilities, logs and the customization panel.
 *     void setCapabilitySettings(Map theCapabilities, AbleSecureKey key)
 *     void setCapabilitySetting(Object capabilityName, int autonomyLevel, AbleSecureKey key)
 *     Map getCapabilities()
 *     Map getCapabilitySettings()
 *     AbleEServerCapability getCapability(Object capabilityName)
 *     Map[] getCapabilityPresets()
 *     AbleMessageContainer getCapabilitySettingsSummary(int autonomyLevel)
 *     AbleMessageContainer getCapabilitySettingsSummary(Map capabilities)
 *     Vector getCapabilityOrder()
 *     void setAgentAutonomyLevel(int agentAutonomyLevel, AbleSecureKey key)
 *     int getAutonomyLevel()
 *     void answerRequest(AbleAgentSituationReport agentReport, AbleSecureKey key)
 *     byte[] getCustomizerResource(String resourceName)
 *     void setCustomizerDataBean(Object dataBean, AbleSecureKey key)
 *     byte[] getCustomizerDataBean()
 *     byte[] getAboutDataBean()
 *     String[] getCustomizerInfo()
 * <p>
 * AbleEServerDefaultAgent extends AblePlatformDefaultAgent which gives you all of the 
 * JAS(Java Agent Specification) specific parts an agent needs to be part of a JAS platform.
 * <p>
 * The Life Cycle Service will use methods from AblePlatformDefaultAgent to perform the
 * actions to an agent:
 *     create
 *     initialize
 *     reinitialize
 *     quit
 *     suspend
 *     reset
 *     resume
 *     getCreatedAgents
 *     quiesce
 * <p>
 * AblePlatformDefaultAgent also holds most of the attributes of an agent:
 *     agentAutonomyLevel 
 *     agentLastUpdate 
 *     agentStatus
 *     agentSummary 
 *     agentType  
 *     agentToAutoRegister
 *     agentToBindToRmi
 *     agentVendor
 *     agentVersion 
 *     agentMovable
 *     agentPrincipal
 *     agentTrustLevel
 *     agentLocator
 *     agentKeyPair
 *     agentName
 *     agentDescrption
 * <p>
 * AblePlatformDefaultAgent also hold the important Jas services and parts:
 *     ServiceRoot
 *     AgentDirectoryService
 *     AgentNamingService
 *     TransportSystem
 * <p>
 * AblePlatformDefaultAgent extends AbleRemoteDefaultAgent which contains the following variables:
 *    agentAddr
 *    agentHost
 *    remoteStub
 * <p>
 * AbleRemoteDefaultAgent extends AbleDefaultAgent which contains the rest of the basic Able 
 * Agent functionality:
 *     defaultName
 *     beans
 *     activeDataSource
 *     eventConnections
 *     processList
 *     processListOK
 *     numEpochs
 *     userDefinedFunctions
 *     inputBufferTargetObject
 *     outputBufferTargetObject
 */
public class EServerTemplateAgent extends AbleEServerDefaultAgent
{   
  //Define the names and identifiers of the capabilities and situations of this agent
  private final Integer TIME_CAPABILITY_ID       = new Integer(1);
  private AbleMessageContainer TIME_MONITOR_CAPABILTIY_NAME = 
    new AbleMessageContainer("AESTA_TimeMonitor",null, AblePlatform.PlatformMessageBundle);
  private final Integer MINUTE_CHANGED_SITUATION_ID   = new Integer(2);
  private AbleMessageContainer MINUTE_CHANGING_SITUATION_NAME =
    new AbleMessageContainer("AESTA_MinuteChanged",null, AblePlatform.PlatformMessageBundle);
  private final Integer HOUR_CHANGED_SITUATION_ID   = new Integer(3); 
  private AbleMessageContainer HOUR_CHANGING_SITUATION_NAME =
    new AbleMessageContainer("AESTA_HourChanged",null, AblePlatform.PlatformMessageBundle);
  private final Integer DUP_CAPABILITY_ID       = new Integer(4);
  private AbleMessageContainer DUP_REQUEST_CAPABILITY_NAME = 
    new AbleMessageContainer("AESTA_DuplicateRequestMonitor", null, AblePlatform.PlatformMessageBundle);
  private final Integer DUP_MINUTE_REQ_SITUATION_ID  = new Integer(5);
  private AbleMessageContainer DUP_MINUTE_REQ_SITUATION_NAME =
    new AbleMessageContainer("AESTA_MinuteDuplicateFound",null, AblePlatform.PlatformMessageBundle);
  private final Integer DUP_HOUR_REQ_SITUATION_ID   = new Integer(6);
  private AbleMessageContainer DUP_HOUR_REQ_SITUATION_NAME =
    new AbleMessageContainer("AESTA_HourDuplicateFound",null, AblePlatform.PlatformMessageBundle);
    
    //Define the responses of the MINUTE_CHANGING situation
  private AbleMessageContainer MINUTE_NO_INFO = new AbleMessageContainer(
    "AESTA_LogMinuteWithoutTellingTime", null, AblePlatform.PlatformMessageBundle);
  private AbleMessageContainer MINUTE_LONG_INFO = new AbleMessageContainer(
    "AESTA_LogMinuteWithTellingTimeAsLong",null, AblePlatform.PlatformMessageBundle);
  private AbleMessageContainer MINUTE_MMDDYYHHMMSS_INFO = new AbleMessageContainer(
    "AESTA_LogMinuteWithTellingTimeInMMDDYY",null, AblePlatform.PlatformMessageBundle);
  
  //Define the no resposne that the MINUTE and HOUR changing situations will use
  private AbleMessageContainer NO_RESPONSE = new AbleMessageContainer(
    "AESTA_DoNothing",null, AblePlatform.PlatformMessageBundle);
    
  //Define the responses of the HOUR_CHANGING situation
  private AbleMessageContainer HOUR_NO_INFO = new AbleMessageContainer(
    "AESTA_LogHourWithoutTellingTime",null, AblePlatform.PlatformMessageBundle);
  private AbleMessageContainer HOUR_LONG_INFO = new AbleMessageContainer(
    "AESTA_LogHourWithTellingTimeAsLong",null, AblePlatform.PlatformMessageBundle);
  private AbleMessageContainer HOUR_MMDDYYHHMMSS_INFO = new AbleMessageContainer(
    "AESTA_LogHourWithTellingTimeInMMDDYY",null, AblePlatform.PlatformMessageBundle);
  
  //Define the responses for the DUPLICATE_MINUTE_REQUEST and DUPLICATE_HOUR_REQUEST situtaions
  private AbleMessageContainer CREATE_DUPLICATE = new AbleMessageContainer(
    "AESTA_CreateDuplicate",null, AblePlatform.PlatformMessageBundle);
  private AbleMessageContainer DO_NOT_CREATE_DUPLICATE = new AbleMessageContainer(
    "AESTA_DoNotCreateDuplicate",null, AblePlatform.PlatformMessageBundle);
  
  //Helps the determine when the minute and hour changes
  private final long MILLISECONDS_PER_MINUTE   = 60000;
  private final long MILLISECONDS_PER_HOUR   = 3600000;
  private long minuteTracker = 0;
  private long hourTracker = 0;   
  
  //The agent will recheck the time using this interval
  private long cycleTime = 10000;
  
  //Used for the summary string of the agent that will be created.
  private long minuteNoTime     = 0;
  private long minuteLongTime   = 0;
  private long minuteFormatTime   = 0;
  private long hourNoTime     = 0;
  private long hourLongTime     = 0;
  private long hourFormatTime   = 0;
  
  //****************
  // Constructors
  //****************
  /**
   * Create this template Time Agent using the defualt name
   * <p>
   * @throws RemoteException On all remote errors.
   * <p>
   * @throws AbleException On errors setting up the capabilities.
   */ 
  public EServerTemplateAgent() throws RemoteException, AbleException
  {
    this("Default Name for EServerTemplateAgent");
  }
   
  /**
   * Create this template Time Agent, setting a name for the agent.
   * <p>
   * @throws RemoteException On all remote errors.
   * <p>
   * @throws AbleException On errors setting up the capabilities.
   */     
  public EServerTemplateAgent(String name) throws RemoteException, AbleException
  {
    super(name);   
    this.setAgentType("Time Monitor");
    //Define the capabilities
    setupCapabilities();
  }
    
  /**
   * Create this template Time Agent, setting a name for the agent, and a comment.
   * <p>
   * @throws RemoteException On all remote errors.
   * <p>
   * @throws AbleException On errors setting up the capabilities.
   */     
  public EServerTemplateAgent(String name, String comment) throws RemoteException, AbleException
  {
    super(name, comment);   
    this.setAgentType("Time Monitor");
    //Define the capabilities
    setupCapabilities();
  }
    
  /**
   * Initialize the basic ABLE agent processing features.  Setup the timer
   * event processing, and reset the variables that will be used to create
   * the summary of this agent.  
   * <p>
   * If <code>theCapabilities</code> are not null, they will be used
   * to set the current capabilities and settings of this agent.
   * <p>
   * @param theCapabilities A Map[] that will be used to set the agents capabilities and settings.
   * <p>
   * @throws On all errors calling the super lclInit
   */
  protected void lclInit(Object capabilities) throws AbleException
  {
    //Says that we will be using timer event processing... 
    setTimerEventProcessingEnabled(true);
    // Sets how frequently the processTimerEvent method will be called
    setSleepTime(cycleTime);
    // We won't be using any event processing   
    setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);
    setDataFlowEnabled(false);
   
    //Reset our tracking variables
    minuteTracker     = 0;
    hourTracker     = 0;
    minuteNoTime     = 0;
    minuteLongTime     = 0;   
    minuteFormatTime   = 0;
    hourNoTime       = 0;
    hourLongTime     = 0;
    hourFormatTime    = 0; 
    startEnabledEventProcessing();
    
    super.lclInit(capabilities);
  }
    
  //************************************************
  // Method overridden from AbleObject
  //************************************************
  /**
   * Since we are using Timer Event processing, this is the method that will be called.
   */
  public synchronized void processTimerEvent() throws AbleException 
  {
    process();   
  }
  
  //************************************************
  // Method overridden from AbleDefaultAgent
  //************************************************
  /**
   * The processTimerEvent method will call this main processing method for this agent.  This is  
   * the method that we will use to determine if an event that our capabilities is monitoring
   * has occured.  If it has, we will send it along to the actOnProblemSituationEncountered method
   * <p>
   * @throws AbleException On all errors thrown from actOnProblemSituationEncountered.
   */
  public void process() throws AbleException
  {
    String lclM = "process";
    
    //Process the Monitor Time capability.  First check if the minute has changed
    long millisecondsPastMinute = System.currentTimeMillis() % MILLISECONDS_PER_MINUTE; 
    if(millisecondsPastMinute<minuteTracker)
    {
      Able.MessageLog.text(Able.MSG_INFO, this, lclM, AblePlatform.NlsMsg("AESTA_MinuteChanged"));
      actOnProblemSituationEncountered(TIME_CAPABILITY_ID,MINUTE_CHANGED_SITUATION_ID);       
    }
    minuteTracker = millisecondsPastMinute;
      
    //Now check if the hour has changed
    long millisecondsPastHour = System.currentTimeMillis() % MILLISECONDS_PER_HOUR; 
    if(millisecondsPastHour<hourTracker)
    {
      Able.MessageLog.text(Able.MSG_INFO, this, lclM, AblePlatform.NlsMsg("AESTA_HourChanged"));
      actOnProblemSituationEncountered(TIME_CAPABILITY_ID,HOUR_CHANGED_SITUATION_ID); 
    }
    hourTracker = millisecondsPastHour;
  }
  
  //***********************************
  //  Methods specific to this class
  //***********************************
  /**
   * This is called by the ctor when the agent is instantiated.  It is the agents
   * responsibility to set the capabilities, and 3 preset Maps with the capability setting.
   * <p>
   * This is where all of the capabilities of an agent are defined.  For this
   * EServerTemplate Time Monitor agent, the first capability, "Monitor Time".
   * will have two situations that could stem from it:  "The minute has changed", and
   * "The hour has changed".  There will be four different responses for each situation
   * that are defined here aswell.
   * <p>
   * The second capability "Duplicate Monitor" will determine if this agent will log duplicate
   * requests.  There will be two different situations, one for each a MINUTE or HOUR change request.
   * <p>
   * Three different settings for this capability will be defined: high, medium and low.
   * The agent will set it's own initial autonomy level to whatever predefined setting
   * it's initial capability settings match.
   * <p>
   * @throws AbleException If the capabilities were not setup correctly  
   */
  private void setupCapabilities() throws AbleException
  {
    //Define the first situation for the "Time Monitor" capability: MINUTE_CHANGING_SITUATION
    AbleMessageContainer[] cap1Situation1Responses = new AbleMessageContainer[]{MINUTE_NO_INFO, MINUTE_LONG_INFO,
                                                                                MINUTE_MMDDYYHHMMSS_INFO, NO_RESPONSE};  
    AbleEServerSituation cap1Situation1 = new AbleEServerSituation(MINUTE_CHANGED_SITUATION_ID, MINUTE_CHANGING_SITUATION_NAME,
      AbleEServerSituation.MEDIUM_SEVERITY, AbleEServerSituation.PERMISSION_REQUEST, cap1Situation1Responses);
                     
     //Define the second situation for the "Time Monitor" capability: HOUR_CHANGING_PROBLEM
    AbleMessageContainer[] cap1Situation2Responses = new AbleMessageContainer[]{HOUR_NO_INFO, HOUR_LONG_INFO,
                                                                                HOUR_MMDDYYHHMMSS_INFO, NO_RESPONSE};
    AbleEServerSituation cap1Situation2 = new AbleEServerSituation(HOUR_CHANGED_SITUATION_ID, HOUR_CHANGING_SITUATION_NAME,
      AbleEServerSituation.LOW_SEVERITY, AbleEServerSituation.PERMISSION_REQUEST, cap1Situation2Responses);
   
    //Create a Map to hold the situations for the time monitor capability.
    HashMap timeCapSituations = new HashMap();
    timeCapSituations.put(MINUTE_CHANGED_SITUATION_ID, cap1Situation1);
    timeCapSituations.put(HOUR_CHANGED_SITUATION_ID, cap1Situation2);
  
    //Define the Time monitoring capability
    AbleEServerCapability timeCapability = new AbleEServerCapability(TIME_CAPABILITY_ID,
      TIME_MONITOR_CAPABILTIY_NAME, AbleEServerCapability.ASK_CAPABILITY, timeCapSituations);

    //Define the first situation for the "Duplicate Monitor" capability: MINUTE_CHANGING_SITUATION
    AbleMessageContainer[] cap2Situation1Responses = new AbleMessageContainer[]{CREATE_DUPLICATE,
                                          DO_NOT_CREATE_DUPLICATE};  
    AbleEServerSituation cap2Situation1 = new AbleEServerSituation(DUP_MINUTE_REQ_SITUATION_ID, DUP_MINUTE_REQ_SITUATION_NAME,
       AbleEServerSituation.MEDIUM_SEVERITY, AbleEServerSituation.PERMISSION_REQUEST, cap2Situation1Responses);
               
    //Define the second situation for the "Duplicate Monitor" capability: HOUR_CHANGING_PROBLEM
    AbleMessageContainer[] cap2Situation2Responses = new AbleMessageContainer[]{CREATE_DUPLICATE,
                                          DO_NOT_CREATE_DUPLICATE};
    AbleEServerSituation cap2Situation2 = new AbleEServerSituation(DUP_HOUR_REQ_SITUATION_ID, DUP_HOUR_REQ_SITUATION_NAME,
      AbleEServerSituation.LOW_SEVERITY, AbleEServerSituation.PERMISSION_REQUEST, cap2Situation2Responses);
   
    //Create a Map to hold the situations for the duplicate request monitor capability.
    HashMap duplicateRequestCapSituations = new HashMap();
    duplicateRequestCapSituations.put(DUP_MINUTE_REQ_SITUATION_ID, cap2Situation1);
    duplicateRequestCapSituations.put(DUP_HOUR_REQ_SITUATION_ID, cap2Situation2);
    
      //Define the duplicate reqeust monitoring capability
    AbleEServerCapability dupReqCapability = new AbleEServerCapability(DUP_CAPABILITY_ID, DUP_REQUEST_CAPABILITY_NAME,
                            AbleEServerCapability.YES_CAPABILITY, duplicateRequestCapSituations);
  
    capabilities.clear();
    capabilities.put(TIME_CAPABILITY_ID, timeCapability);
    capabilities.put(DUP_CAPABILITY_ID, dupReqCapability);
   
    //Define the presets
    highCapabilitySettings.put(TIME_CAPABILITY_ID, new Integer(AbleEServerCapability.YES_CAPABILITY));
    highCapabilitySettings.put(DUP_CAPABILITY_ID, new Integer(AbleEServerCapability.YES_CAPABILITY));
    mediumCapabilitySettings.put(TIME_CAPABILITY_ID, new Integer(AbleEServerCapability.ASK_CAPABILITY));
    mediumCapabilitySettings.put(DUP_CAPABILITY_ID, new Integer(AbleEServerCapability.ASK_CAPABILITY));
    lowCapabilitySettings.put(TIME_CAPABILITY_ID, new Integer(AbleEServerCapability.NO_CAPABILITY));
    lowCapabilitySettings.put(DUP_CAPABILITY_ID, new Integer(AbleEServerCapability.NO_CAPABILITY));
  
    //For checking purposes, and to init the custom capability settings, verify the capabilities
    verifyCapabilities();
  }  
  
  /**
   * When it is determined that a problem situation has occured, this method will look at the capability, 
   * and depending on what autonomy level it is currently set to, will act on the problem situation.
   * <p>
   * @param capabiltiyName The capability that has encountered a problem situation.
   * <p>
   * @param situationName The situation of this capability.
   * <p>
   * @throws AbleException
   */
  private synchronized void actOnProblemSituationEncountered(Object capabilityIdentifier,
                              Object situationIdentifer) throws AbleException
  {
    String lclM="actOnProblemSituationEncountered";       
    //Begin to create the AbleAgentSituation report for this capability's situation
    AbleEServerCapability capability = (AbleEServerCapability)capabilities.get(capabilityIdentifier);
    AbleEServerSituation situation = (AbleEServerSituation)capability.getSituations().get(situationIdentifer);
    AbleAgentSituationReport asr = new AbleAgentSituationReport((Serializable)null,
                                                                capability,
                                                                situation,
                                                                situation.getYesResponse(),
                                                                myJasAgentName,  
                                                                getName(),
                                                                System.currentTimeMillis(),
                                                                getAgentHost(),
                                                                new Integer(getAgentAutonomyLevel())); 
       
    //Check if this request is a duplicate(a similar problem situation queued up waiting for a response)
    AbleAgentSituationReport searchAsr = new AbleAgentSituationReport((Serializable)null,
                                      capability,
                                      situation,
                                      situation.getYesResponse(),
                                      myJasAgentName,  
                                      getName(),
                                      0,  //wildcard the time
                                      getAgentHost(),
                                      new Integer(getAgentAutonomyLevel()));
    //It must be a problem state.
    searchAsr.setProblemState(true);
    Vector found = new Vector();  
    try
    {                                    
      found = myEServerLoggingService.search(searchAsr);
    }
    catch(AgentLoggingException ale)
    {
      throw new AbleException(ale.getLocalizedMessage());
    }
    if(!found.isEmpty())  
    {
      //One was found, now turn to our Duplicate Capability to determine how to handle it.
      AbleEServerCapability dupCapability = (AbleEServerCapability)capabilities.get(DUP_CAPABILITY_ID);
      Object dupSituationIdentifer = (situationIdentifer == HOUR_CHANGED_SITUATION_ID)?DUP_HOUR_REQ_SITUATION_ID:
                                                 DUP_MINUTE_REQ_SITUATION_ID;
      AbleEServerSituation dupSituation = (AbleEServerSituation)dupCapability.getSituations().get(dupSituationIdentifer);
    
      //If set to ASK, log a request.  
      if(dupCapability.getAutonomyLevel() == AbleEServerCapability.ASK_CAPABILITY)
      {
        AbleAgentSituationReport dupAsr = new AbleAgentSituationReport(asr,
                                      dupCapability,
                                      dupSituation,
                                      dupSituation.getYesResponse(),
                                      myJasAgentName,  
                                      getName(),
                                      System.currentTimeMillis(),
                                      getAgentHost(),
                                      new Integer(getAgentAutonomyLevel())); 
        //Log the dup request, and then another history logs saying that a problem had occured
        logAbleAgentSituationReport(dupAsr);
        logHistory(new AbleMessageContainer("AESDA_RequestHasBeenLogged", 
                new Object[]{getName()}, AblePlatform.PlatformMessageBundle),
               new AbleMessageContainer("AESDA_RequestDescription",
                new Object[]{dupAsr.getCapability().getName(),dupAsr.getSituation().getName()},
                AblePlatform.PlatformMessageBundle),
               new AbleMessageContainer("AESDA_WillBeWaitingForResponse",new Object[]{getName()},
                AblePlatform.PlatformMessageBundle));
        return;
      }
      //If set to NO, do not create the dup.
      else if(dupCapability.getAutonomyLevel() == AbleEServerCapability.NO_CAPABILITY)
        return;
      //If set to YES, log the dup request.... continue
    }

    //when the capability is set to YES
    if(capability.getAutonomyLevel() == AbleEServerCapability.YES_CAPABILITY)
    {
      //Log a history log telling that the agent is acting on it's own
      logHistory(new AbleMessageContainer("AESDA_AgentActingOnOwn",
                 new Object[]{getName(),new AbleMessageContainer("ASR_YES",null,
                 AblePlatform.PlatformMessageBundle)},AblePlatform.PlatformMessageBundle),
                 new AbleMessageContainer("ASR_CapabilitySetting",
                 new Object[]{capability.getName(),new AbleMessageContainer("ASR_YES",
                 null,AblePlatform.PlatformMessageBundle)},AblePlatform.PlatformMessageBundle),
                 new AbleMessageContainer("ASR_WillBeTakingResponse",
                 new Object[]{situation.getYesResponse()},AblePlatform.PlatformMessageBundle));
         
      asr.setProblemState(true);
      asr.setResponseTaken(situation.getYesResponse());
      doAction(situationIdentifer, asr, false);
      return;
    }         
    //when the capability is set to NO
    else if(capability.getAutonomyLevel() == AbleEServerCapability.NO_CAPABILITY)
    {
      //Log a history log telling that the agent is acting on it's own
      logHistory(new AbleMessageContainer("AESDA_AgentActingOnOwn",
                new Object[]{getName(),new AbleMessageContainer("ASR_NO",null,
                AblePlatform.PlatformMessageBundle)},AblePlatform.PlatformMessageBundle),
                new AbleMessageContainer("ASR_CapabilitySetting",
                new Object[]{capability.getName(),new AbleMessageContainer("ASR_NO",
                null,AblePlatform.PlatformMessageBundle)},AblePlatform.PlatformMessageBundle),
                new AbleMessageContainer("ASR_WillBeTakingResponse",
                new Object[]{situation.getNoResponse()},AblePlatform.PlatformMessageBundle));
         
       asr.setProblemState(false);
       asr.setResponseTaken(situation.getNoResponse());
       asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentActingOnOwn",
                   new Object[]{getName(),new AbleMessageContainer("ASR_NO",null,
                   AblePlatform.PlatformMessageBundle)}, AblePlatform.PlatformMessageBundle));
      logAbleAgentSituationReport(asr);         
    }         
    //when the capability is set to ASK
    else
    {
      asr.setProblemState(true);
      //Log the actual request, and then another history logs saying that a problem had occured
      logAbleAgentSituationReport(asr);
      logHistory(new AbleMessageContainer("AESDA_RequestHasBeenLogged", 
                 new Object[]{getName()}, AblePlatform.PlatformMessageBundle),
                 new AbleMessageContainer("AESDA_RequestDescription",
                 new Object[]{asr.getCapability().getName(),asr.getSituation().getName()},
                 AblePlatform.PlatformMessageBundle),
                 new AbleMessageContainer("AESDA_WillBeWaitingForResponse",new Object[]{getName()},
                 AblePlatform.PlatformMessageBundle));
    }
  }
     
  /**
   * This is the method that does the actaul work when handling a problem situation.  This could be
   * called by ourselves if we came across a problem situation, and the autonomy level of that capability
   * was set to YES.  This could also be called if an answer to one of this agent's previously
   * logged Requests had been answered using the answerRequest method.
   * <p>
   * @param situationIdentifier The identifier of the situation that this aciton will be fixing.
   * <p>
   * @param asr The AbleAgentSituationReport that contain the information on how to deal with this situation
   * <p>
   * @param isFromAnsweredRequest Was this request answered internally, or externally?
   * <p>
   * @throws AbleException
   * <p>On errors taking the action
   * <p>If <code>situationIdentifier</code> does not match one of this agent's situations.
   */
  private void doAction(Object situationIdentifier, AbleAgentSituationReport asr, boolean isFromAnsweredRequest)
                                                 throws AbleException
  {
    //Depending on the situation ID, take the appropriate action
    //If this is from our minute changed situation
    if(situationIdentifier.equals(MINUTE_CHANGED_SITUATION_ID))
      actOnMinuteChanged(asr, isFromAnsweredRequest);
    //If this is from out hour changed situation  
    else if(situationIdentifier.equals(HOUR_CHANGED_SITUATION_ID))
      actOnHourChanged(asr, isFromAnsweredRequest);
    //If this is from out dup request capability  
    else if(situationIdentifier.equals(DUP_HOUR_REQ_SITUATION_ID)||(situationIdentifier.equals(DUP_MINUTE_REQ_SITUATION_ID)))
      actOnDupReqFound(asr);
    //This isn't from one of our situations...
    else   
      throw new AbleException(AblePlatform.NlsMsg("AESDA_AgentCannotRespondToUnknownASR",new Object[]{situationIdentifier}));
  }
  
  /**
   * This method handles the dup request found event of the "Duplicate Monitor" capability.
   * <p>
   * By now we know that a dup was found.  This method will 
   * <p>
   * @param asr The AbleAgentSituationReport that says how to handle this situation
   * <p>
   * @throws AbleException
   * <p>On errors updating the log in the logging service.
   * <p>On errors updating the agent's summary string.
   */
  private void actOnDupReqFound(AbleAgentSituationReport asr) throws AbleException
  {
    //The situation of this capability has 2 responses.
    if(asr.getResponseTaken().equals(DO_NOT_CREATE_DUPLICATE))  //Do not create a duplicate request
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                new Object[]{getName(),DO_NOT_CREATE_DUPLICATE, "-"}, AblePlatform.PlatformMessageBundle));
    else if(asr.getResponseTaken().equals(CREATE_DUPLICATE))  //Create a duplicate request
    {
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
              new Object[]{getName(),CREATE_DUPLICATE, "-"}, AblePlatform.PlatformMessageBundle));
      logAbleAgentSituationReport((AbleAgentSituationReport)asr.getIdentifier());
    }
    else  //invalid response taken in the Agent Situation Report
      asr.setResultsSummary(new AbleMessageContainer("AESDA_SuggestedResponseInvalid",
                  new Object[]{asr.getResponseTaken()},AblePlatform.PlatformMessageBundle));
       
    //Update the asr with the logging service
    updateAsrAfterAction(asr);
  }
   
  /**
   * This method handles the minute changing event of the "Time Monitor" capability
   * By now, the asr should be set to know what action is to be taken (by the console, this agent,
   * or another agent.  This is where the action will happen.  Since this is a simple agent, the 
   * action is only differnt ways of loggin this information as history to the logging service.
   * <p>
   * @param asr The AbleAgentSituationReport that says how to handle this situation
   * <p>
   * @param isFromAnsweredRequest Was this request answered internally, or externally?
   * <p>
   * @throws AbleException
   * <p>On errors updating the log in the logging service.
   * <p>On errors updating the agent's summary string.
   */
  private void actOnMinuteChanged(AbleAgentSituationReport asr, boolean isFromAnsweredRequest) throws AbleException
  {
    //The situation of this capability has 3 responses.
    if(asr.getResponseTaken().equals(MINUTE_NO_INFO))  //Send no time info in the asr
    {
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                   new Object[]{getName(),MINUTE_NO_INFO, "-"}, AblePlatform.PlatformMessageBundle));
      //Update our tracking info for the summary string we create
      minuteNoTime++;
    }
    else if(asr.getResponseTaken().equals(MINUTE_LONG_INFO)) //Send the time info as a long
    {
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                     new Object[]{getName(),MINUTE_LONG_INFO, new Long(asr.getTimeCreated())},
                     AblePlatform.PlatformMessageBundle));  
         //Update our tracking info for the summary string we create
      minuteLongTime++;
    }
    else if(asr.getResponseTaken().equals(MINUTE_MMDDYYHHMMSS_INFO))  //Send the time info as MMDDYY
    {
      SimpleDateFormat formatter = new SimpleDateFormat ("MM/dd/yyyy hh:mm:ss a");
      Date timeSent = new Date(asr.getTimeCreated());
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                   new Object[]{getName(),MINUTE_MMDDYYHHMMSS_INFO,formatter.format(timeSent)},
                   AblePlatform.PlatformMessageBundle));
      //Update our tracking info for the summary string we create
      minuteFormatTime++;
    }
    else if(asr.getResponseTaken().equals(NO_RESPONSE))  //No response is taken, ignore the problem
    {
       AbleAgentSituationReport asrNew = new AbleAgentSituationReport(asr);
       asrNew.setProblemState(false);
       asr.setResponseTaken(null);
       updateAbleAgentSituationReport(asr,asrNew);
       return;
    }
    else  //invalid response taken in the Agent Situation Report
      asr.setResultsSummary(new AbleMessageContainer("AESDA_SuggestedResponseInvalid",
                     new Object[]{asr.getResponseTaken()},AblePlatform.PlatformMessageBundle));
         
    //If this was answered externally, update the asr with the logging service
    if(isFromAnsweredRequest)
      updateAsrAfterAction(asr);
    else
    {
      asr.setProblemState(false);
      logAbleAgentSituationReport(asr);
    }
       
    //update the summary string of this agent to reflect what it has done
    updateAgentSummaryString();
  }
   
  /**
   * This method handles thehour changing event of the "Time Monitor" capability
   * By now, the asr should be set to know what action is to be taken (by the console, this agent,
   * or another agent.  This is where the action will happen.  Since this is a simple agent, the 
   * action is only differnt ways of loggin this information as history to the logging service.
   * <p>
   * @param asr The AbleAgentSituationReport that says how to handle this situation
   * <p>
   * @throws AbleException
   * <p>
   * @param isFromAnsweredRequest Was this request answered internally, or externally?
   * <p>
   * @throws AbleException
   * <p>On errors updating the log in the logging service.
   * <p>On errors updating the agent's summary string.
   */
  private void actOnHourChanged(AbleAgentSituationReport asr, boolean isFromAnsweredRequest) throws AbleException
  {
    //The situation of this capability has 3 responses.
    if(asr.getResponseTaken().equals(HOUR_NO_INFO))  //Send no time info in the asr
    {
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                 new Object[]{getName(),HOUR_NO_INFO, "-"},AblePlatform.PlatformMessageBundle));
      //Update our tracking info for the summary string we create
      hourNoTime++;
    }
    else if(asr.getResponseTaken().equals(HOUR_LONG_INFO)) //Send the time info as a long
    {
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                        new Object[]{getName(),HOUR_LONG_INFO, new Long(asr.getTimeCreated())},
                        AblePlatform.PlatformMessageBundle));  
      //Update our tracking info for the summary string we create
      hourLongTime++;
    }
    else if(asr.getResponseTaken().equals(HOUR_MMDDYYHHMMSS_INFO))  //Send the time info as MMDDYY
    {
      SimpleDateFormat formatter = new SimpleDateFormat ("MM/dd/yyyy hh:mm:ss a");
      Date timeSent = new Date(asr.getTimeCreated());
      asr.setResultsSummary(new AbleMessageContainer("AESDA_AgentTookActionBasedOffResponse",
                 new Object[]{getName(),MINUTE_MMDDYYHHMMSS_INFO, formatter.format(timeSent)},
                 AblePlatform.PlatformMessageBundle));
      //Update our tracking info for the summary string we create
      hourFormatTime++;
    }
    else if(asr.getResponseTaken().equals(NO_RESPONSE))  //No response is taken, ignore the problem
    {
      AbleAgentSituationReport asrNew = new AbleAgentSituationReport(asr);
      asrNew.setProblemState(false);
      asr.setResponseTaken(null);
      updateAbleAgentSituationReport(asr,asrNew);
      return;
    }
    else  //invalid response taken in the Agent Situation Report
      asr.setResultsSummary(new AbleMessageContainer("AESDA_SuggestedResponseInvalid",
                   new Object[]{asr.getResponseTaken()}, AblePlatform.PlatformMessageBundle));
         
    //If this was answered externally, update the asr with the logging service
    if(isFromAnsweredRequest)
      updateAsrAfterAction(asr);
     
       //update the summary string of this agent to reflect what it has done
       updateAgentSummaryString();
  }
   
  /**
   * After actions have been taken, this method has the Logging Service update this ASR.
   * This should only be called when this agent is taking action because of an externally
   * answered AbleAgentSituationReport that exists in the LoggingService.
   * <p> 
   * At this point, the only thing that should be different in the ASR from the logging 
   * service is the result summary field, and responseTaken.
   * <p>
   * @param asr The AbleAgentSituationReport that we will have the Logging Service update
   * <p>
   * @throws AbleException On errors updating the AbleAgentSituationReport in the Logging Service.   
   */
  private void updateAsrAfterAction(AbleAgentSituationReport asr) throws AbleException
  {
    String lclM = "updateAsrAfterAction";
    //Create a new asr that will represent the old ASR that should still exist in the logging service
    AbleAgentSituationReport oldAsr = new AbleAgentSituationReport(asr);
    oldAsr.setResultsSummary(null);
    oldAsr.setResponseTaken(null);

    //Mark the new ASR as being a history log now that we've had it answered
    asr.setProblemState(false);
       
    //Update this ASR in the Logging Service
    updateAbleAgentSituationReport(oldAsr,asr);       
  }
  
  /**
   * Part of the AblePlatformDefaultAgent is the summary of the agent.  This is just an quick overview
   * of what the agent has done.  Whenever we do something, this should be updated, so when someone
   * at the console is viewing the agent, they can look at the summary and get an idea of what this
   * agent has done
   * <p>
   * @throws AbleException
   * <p>On errors getting the current summary String from this agent.
   * <p>On errors updating the agent description with this new summary String.
   */
  private void updateAgentSummaryString() throws AbleException
  {
    setAgentSummary(new AbleMessageContainer("AESTA_SummaryString", new Object[]{new Long(minuteNoTime),
      new Long(hourNoTime), new Long(minuteLongTime), new Long(hourLongTime), new Long(minuteFormatTime),
      new Long(hourFormatTime)}, AblePlatform.PlatformMessageBundle));
    
    //Have the agent update its description with this new information.
    myJasAgentDescription.set(AblePlatform.AbleAgentSummary, getAgentSummary());
    updateDescriptionWithAgentDirectoryService();    
  }
  
  //*********************************************************
  //  Methods overridden from AbleEServerDefaultAgent
  //*********************************************************  
  /**
   * This method is what handles authenticating and authorizing the caller who is answering one of the
   * agent's logged situations.
   * <p>  
   * @param asr AbleAgentSituationReport that is the answer to one of the agents logged situations
   * <p>
   * @param key The AbleSecureKey the answering entity used to answer this situation
   * <p>
   * @throws AbleException If the caller is not authorized.
   */
  protected void authenticateAndAuthorizeCaller(AbleAgentSituationReport asr, AbleSecureKey key)
                                            throws AbleException
  {
    String lclM = "authenticateAndAuthorizeCaller";
    //First, it is up to this agent to authenticate and authorize the VAN, and decide if it should do
    //what it has asked.  Since this agent does such important tasks, if it is running secure, we will
    //make sure that whoever is anwering it's requests has at least similar authority
    if(Able.Preferences.isSecure())
    {
      //The VAN have already been verified, but authorize the call chain of the key.
      try
      {
        mySecSppt.verifyKey(key);
        mySecSppt.verifyTrustLevel(key,lclM);
      }
      catch(Exception e)
      {
        //Create the AbleMessageContainers to be the description, action and summary of the log
        AbleMessageContainer description = new AbleMessageContainer(e.getLocalizedMessage());
        AbleMessageContainer action = new AbleMessageContainer("AESDA_RequestDescription",
                    new Object[]{asr.getCapability().getName(),
                    asr.getSituation().getName()}, AblePlatform.PlatformMessageBundle);
        AbleMessageContainer summary = new AbleMessageContainer("AESDA_ResponseNotTaken",
                    new Object[]{asr.getResponseTaken()}, AblePlatform.PlatformMessageBundle);
        try
        {
          //This is not the exception that we want to be throwing.
          logHistory(description,action,summary);
        }
        catch(Exception e2)
        {/* This was already logged inside of logHistory*/}              
          
        if (Able.TraceLog.isLogging()) logger.text(Able.MSG_ERROR, this, lclM, e.getLocalizedMessage());
            throw new AbleException(e.getLocalizedMessage());
      }                                                                             
    }
  }
     
  /**
   * This method will decide if this request is still a valid request that help is needed on.
   * If the request is an hour changing event, the request will not longer be valid if it is 
   * more than 6 minutes old.  If it is a minute changing event, it will no longer be valid
   * if it is more than 3 minutes old.
   * <p>
   * @param reportToValidate The AbleAgentSituationReport that is to be validated.
   * <p>
   * @throws AbleException When the request is no longer valid.  
   */
  protected void validateRequest(AbleAgentSituationReport reportToValidate) throws AbleException
  {
    String lclM = "validateRequest";
      
    //A duplicate request is always valid
    if(reportToValidate.getSituation().getSituationIdentifier().equals(DUP_HOUR_REQ_SITUATION_ID)||
    reportToValidate.getSituation().getSituationIdentifier().equals(DUP_MINUTE_REQ_SITUATION_ID))
      return;
        
    //If a minute event, the log must not be older than three minutes, is an hour, six minutes.
    long expireTime = 0;
      
    if(reportToValidate.getSituation().getSituationIdentifier().equals(MINUTE_CHANGED_SITUATION_ID))
      expireTime = 180000;  // 3 minutes (1000*60*3) = 180000 ms
    else if(reportToValidate.getSituation().getSituationIdentifier().equals(HOUR_CHANGED_SITUATION_ID))
      expireTime = 360000;  // 6 minutes (1000*60*6) = 360000 ms
      
    if(reportToValidate.getTimeCreated()<(System.currentTimeMillis()-expireTime))
    {
      //Create the AbleMessageContainers to be the description, action and summary of the log
      AbleMessageContainer description = new AbleMessageContainer("AESTA_RequestNotValidated",
          new Object[]{reportToValidate.getSituation().getName()}, AblePlatform.PlatformMessageBundle);
      AbleMessageContainer action = new AbleMessageContainer("AESDA_RequestDescription",
          new Object[]{reportToValidate.getCapability().getName(),
          reportToValidate.getSituation().getName()}, AblePlatform.PlatformMessageBundle);
      AbleMessageContainer summary = new AbleMessageContainer("AESDA_ResponseNotTaken",
          new Object[]{reportToValidate.getResponseTaken()}, AblePlatform.PlatformMessageBundle);                        
      try
      {
        //This is not the exception that we want to be throwing
        logHistory(description,action,summary);
      }
      catch(Exception e)
      {/* This was already logged inside of logHistory*/}                        
      
      AbleMessageContainer error = new AbleMessageContainer("AESTA_RequestNotValidated",
            new Object[]{reportToValidate.getSituation().getName()},AblePlatform.PlatformMessageBundle);
      String lclMsg = AblePlatform.NlsMsg(error.getMessageID(), error.getSubText());
      if (Able.TraceLog.isLogging()) logger.text(Able.MSG_ERROR, this, lclM, lclMsg);
          throw new AbleException(error, lclMsg);      
    }
  }
  
  /**
   * When an AbleAgentSituationReport comes into this agent's answerRequest method, if secure, the
   * AbleAgentSituationReport is first authenticated, and then authorized.  Once past those checks,
   * an AbleEServerAgentAnswerThread is created with this AbleAgentSituationReport passed into it.
   * This new thread will call this _answerRequest method which will process the answer.
   * <p>
   * Any failures in answering the request that came back to this method will get logged to the Logging
   * Service as a History Log.
   * <p>  
   * @param agentReport The answered AbleAgentSituationReport that this agent had previoulsy logged.
   * <p>
   * @param key The AbleSecureKey that was used to answer this request
   */
  protected void _answerRequest(AbleAgentSituationReport agentReport, AbleSecureKey key)
  {
    String lclM = "_answerRequest";
    //send to doAction so action can be taken, and the log updated.
    try
    {
      //Send a history log telling that someone is attemting to answer one of our requests
      AbleMessageContainer requestor = (key==null?new AbleMessageContainer("AESDA_RequestorWhenSecurityIsOff", 
                    null, AblePlatform.PlatformMessageBundle): 
                    new AbleMessageContainer(key.getVan().getVanOfInitiator().getKerbPrincipal()));
      AbleMessageContainer description = new AbleMessageContainer("AESDA_RequestorTryingToAnswerRequest",
                   new Object[]{requestor, getName()}, AblePlatform.PlatformMessageBundle);
      AbleMessageContainer action = new AbleMessageContainer("AESDA_RequestDescription",
              new Object[]{agentReport.getCapability().getName(),agentReport.getSituation().getName()},
              AblePlatform.PlatformMessageBundle);                   
      AbleMessageContainer summary = new AbleMessageContainer("AESDA_ResponseTaken",
              new Object[]{agentReport.getResponseTaken()}, AblePlatform.PlatformMessageBundle);
      logHistory(description, action, summary);
             
      doAction(agentReport.getSituation().getSituationIdentifier(), agentReport, true);    
    }
    //An error occured answering this situation... log a history log describing this error
    catch(AbleException re)
    {
      logException(lclM, re);
      AbleMessageContainer description = new AbleMessageContainer("AESDA_ErrorProcessingAnswer",
            new Object[]{getName()}, AblePlatform.PlatformMessageBundle);
      AbleMessageContainer action = new AbleMessageContainer("AESDA_RequestDescription",
        new Object[]{agentReport.getCapability().getName(), agentReport.getSituation().getName()},
        AblePlatform.PlatformMessageBundle);
      AbleMessageContainer summary = new AbleMessageContainer("AESDA_ProcessingFailure", null,
                                  AblePlatform.PlatformMessageBundle);
      try
      {
        logHistory(description,action,summary);
      }
      catch(Exception e)
      {/* This was already logged inside of logHistory*/}
    }       
  }
     
  /**
   * This method is what handles authorizing a capability change request to this agent.  When secure, 
   * this agent will only let someone change it's capabilities if it's level is greater than or equal to
   * the level of this agent.  This assumes the key has already been verified.
   * <p>
   * @param key The AbleSecureKey of the entity trying to make the change request.
   * <p>
   * @throws AbleException If the caller is not authorized.
   */
  protected void authorizeCapabilityChangeRequest(AbleSecureKey key) throws AbleException
  {
    String lclM = "authorizeCapabilityChangeRequest";
    //First, it is up to this agent to authorize this capability change request if it is running 
    //secure, we will make sure that whoever is anwering it's requests has at least similar authority
    if(Able.Preferences.isSecure())
    {
      //If not authorized, send a log telling why
      if(AblePlatform.Preferences.compareTrustLevel(myVerifiableAgentName.getLevel(),
                                         AblePlatformPreferences.TRUSTLEVEL_GREATER_THAN,key.getVan().getLevel()))
      {
        //Create the AMCs for the history log
        AbleMessageContainer description = new AbleMessageContainer("AESDA_CapChangeRequestNotAuthorized",
                new Object[]{key.getVan().getName(), key.getVan().getLevel(), getName(),
                myVerifiableAgentName.getLevel()}, AblePlatform.PlatformMessageBundle);
        AbleMessageContainer summary = new AbleMessageContainer("AESDA_CapabilityChangeRequestNotTaken",
                                    null, AblePlatform.PlatformMessageBundle);
        try
        {
          //This is not the exception that we want to be throwing.
          logHistory(description,new AbleMessageContainer(""),summary);
        }
        catch(Exception e)
        {/* This was already logged inside of logHistory*/}
        
        //Create the error AMC, use it to help log to the trace, and then to throw an AbleException.      
        AbleMessageContainer error = new AbleMessageContainer("CallerTrustLevelTooLowToRunMethod",
                new Object[]{key.getVan().getKerbPrincipal(), key.getVan().getLevel(), lclM,
                myVerifiableAgentName.getLevel()}, AblePlatform.PlatformMessageBundle);  
        String lclMsg = AblePlatform.NlsMsg(error.getMessageID(), error.getSubText());
        if (Able.TraceLog.isLogging()) logger.text(Able.MSG_ERROR, this, lclM, lclMsg);
        throw new AbleException(error, lclMsg);
      }                                                    
    }
  }  
     
  /**
   * Return a String [] array with the following information:
   *
   *     --Return null for Strings 0,1, and 2 if there is no Customizer panel--
   * 
   *     [0]  - Name of Customizer AUIML file and resource bundle class. E.g.:
   *            "com.ibm.agent.CustomizerPanel"
   *     [1]  - The Customizer data bean class to use with the AUIML panel.  The 
   *            _setCustomizerData() method should be expecting a class
   *            of this type. e.g.: "com.ibm.agent.CustomizerDataBean"
   *     [2]  - Name of Customizer panel in auiml file. E.g.: "MainPanel"
   * 
   *     --The following three Strings are only required if there is an About panel--
   * 
   *     [3]  - Name of AUIML file that has the About panel. E.g.:
   *            "com.ibm.agent.AboutPanel"
   *     [4]  - The About data bean class to use with the AUIML panel.
   *            E.g.: "com.ibm.agent.AboutDataBean"
   *     [5]  - Name of About panel in auiml file. E.g.: "AboutPanel"
   * <p>
   * Example:  myString [0] = "com.ibm.agent.CustomizerPanel"
   *           myString [1] = "com.ibm.agent.CustomizerDataBean"
   *           myString [2] = "MainPanel"
   * <p>
   * Default implementation is to return null.  This tells the console that the agent doesn't
   * have a customizer panel.
   * <p>
   * @return String []
   */
  protected String [] _getCustomizerInfo()
  {
    return new String [] {"com.ibm.able.platform.examples.EServerTemplateAgentCustomizer",
                          "com.ibm.able.platform.examples.EServerTemplateAgentDataBean",
                          "ESTACustomizer",
                          "com.ibm.able.platform.examples.EServerTemplateAgentAboutPanel",
                          "com.ibm.able.platform.examples.EServerTemplateAgentAboutDataBean",
                          "ESTAAbout"};
  }

  /**
   * The user will enter values into the customizer panel on the client, and the
   * AUIML data bean will get serialized and sent to this agent with this method.
   * <p>
   * @param dataBean The dataBean that is used to customize this agent.
   * <p>
   * @param key AbleSecureKey used to secure the call.
   * <p>
   * @throws AbleException On all errors authenticiating and authorizing the caller.
   */
  protected void _setCustomizerDataBean(Object dataBean, AbleSecureKey key) throws AbleException
  {
    //If secure, the call must have equal or greater authority than this agent.
    if(Able.Preferences.isSecure())
    {
      try
      {
        //Authenticate the caller
        mySecSppt.verifyKey(key);  
        //Authorize the caller, they must have an equal or greater trust level
        mySecSppt.verifyTrustLevel(key, "_setCustomizerDataBean");
      }
      catch(NamingException ne)
      {
        throw new AbleException(ne.getLocalizedMessage());
      }
    }
    
    long oldSleepTime = getSleepTime();
    long newSleepTime = (long)((EServerTemplateAgentDataBean)dataBean).getcycleTime();
    cycleTime = newSleepTime;
    setSleepTime(cycleTime);
    
    //Send a history log about this change.
    AbleMessageContainer description = new AbleMessageContainer("AESDA_SetCustomizerDataBean",
        new Object[]{getName()}, AblePlatform.PlatformMessageBundle);
    AbleMessageContainer action = new AbleMessageContainer("");
    AbleMessageContainer summary = new AbleMessageContainer("AESDA_CycleTimeChanged",
        new Object[]{new Long(oldSleepTime), new Long(newSleepTime)}, AblePlatform.PlatformMessageBundle);
    try
    {
      logHistory(description,action,summary);
    }
    catch(Exception e)
    {/* This was already logged inside of logHistory*/}  
  }

  /**
   * Get the databean for this agent.  It can then be changed and sent back to the agent
   * through the _setCustomizerDataBean method.
   * <p>
   * @return The dataBean that can be used to customize this agent.
   * <p>
   * @throws AbleException On any error creating the bean.
   */
  protected Object _getCustomizerDataBean() throws AbleException
  {
    EServerTemplateAgentDataBean dataBean = new EServerTemplateAgentDataBean();
    dataBean.setcycleTime(cycleTime);
    
    return dataBean;
  }    
  
    /**
     * Get the about databean for this agent.  It is a simple bean that will
     * describe what this agent does.
     * <p>
     * @return The dataBean that can be used to describe this agent.
     * <p>
     * @throws AbleException On any error creating the bean.
     */
    protected Object _getAboutDataBean() throws AbleException
    {
      EServerTemplateAgentAboutDataBean dataBean = new EServerTemplateAgentAboutDataBean();
      // TODO add "AESDA_AboutMessage" to the message bundle in V5R4
      dataBean.setAboutMessage("The AbleEServerTemplateAgent is a simple template agent that can be used "+
                               "as a tutorial when creating your own agents that are to be managed through "+
                               "the Intelligent Agents console.  It demonstrates how an agent would create "+
                               "a set of capabilities that could be managed through the console.  It includes "+
                               "a customization panel that can be used to alter agent settings and an about "+
                               "panel that will display information about the agent.  It also shows how an "+
                               "agent would use the Logging Service to log requests and history entries that "+
                               "can be displayed and responded to through the console.  It also makes use of "+
                               "many of the features available when developing agents using the ABLE framework.");
      return dataBean;
    }    
}